﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyDatabaseImplement.Implements
{
    public class WorkpieceStorage : IWorkpieceStorage
    {
        public List<WorkpieceViewModel> GetFullList()
        {
            using var context = new FurnitureAssemblyDatabase();
            return context.Workpieces
            .Select(x => x.GetViewModel)
                .ToList();
        }

        public List<WorkpieceViewModel> GetFilteredList(WorkpieceSearchModel model)
        {
            if (string.IsNullOrEmpty(model.WorkpieceName))
            {
                return new();
            }

            using var context = new FurnitureAssemblyDatabase();

            return context.Workpieces
            .Where(x => x.WorkpieceName.Contains(model.WorkpieceName))
            .Select(x => x.GetViewModel)
                .ToList();
        }

        public WorkpieceViewModel? GetElement(WorkpieceSearchModel model)
        {
            if (string.IsNullOrEmpty(model.WorkpieceName) && !model.Id.HasValue)
            {
                return null;
            }

            using var context = new FurnitureAssemblyDatabase();

            return context.Workpieces
                .FirstOrDefault(x => (!string.IsNullOrEmpty(model.WorkpieceName) && x.WorkpieceName == model.WorkpieceName) ||
                    (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public WorkpieceViewModel? Insert(WorkpieceBindingModel model)
        {
            var newWorkpiece = Workpiece.Create(model);

            if (newWorkpiece == null)
            {
                return null;
            }

            using var context = new FurnitureAssemblyDatabase();
            context.Workpieces.Add(newWorkpiece);
            context.SaveChanges();

            return newWorkpiece.GetViewModel;
        }

        public WorkpieceViewModel? Update(WorkpieceBindingModel model)
        {
            using var context = new FurnitureAssemblyDatabase();
            var workPiece = context.Workpieces.FirstOrDefault(x => x.Id == model.Id);

            if (workPiece == null)
            {
                return null;
            }
            workPiece.Update(model);
            context.SaveChanges();

            return workPiece.GetViewModel;
        }

        public WorkpieceViewModel? Delete(WorkpieceBindingModel model)
        {
            using var context = new FurnitureAssemblyDatabase();
            var element = context.Workpieces.FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                context.Workpieces.Remove(element);
                context.SaveChanges();

                return element.GetViewModel;
            }

            return null;
        }
    }
}
