﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyFileImplement.Implements
{
    public class WorkpieceStorage : IWorkpieceStorage
    {
        private readonly DataFileSingleton source;

        public WorkpieceStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public List<WorkpieceViewModel> GetFullList()
        {
            return source.Workpieces.Select(x => x.GetViewModel).ToList();
        }

        public List<WorkpieceViewModel> GetFilteredList(WorkpieceSearchModel model)
        {
            if (string.IsNullOrEmpty(model.WorkpieceName))
            {
                return new();
            }

            return source.Workpieces.Where(x => x.WorkpieceName.Contains(model.WorkpieceName)).Select(x => x.GetViewModel).ToList();
        }

        public WorkpieceViewModel? GetElement(WorkpieceSearchModel model)
        {
            if (string.IsNullOrEmpty(model.WorkpieceName) && !model.Id.HasValue)
            {
                return null;
            }

            return source.Workpieces.FirstOrDefault(x => (!string.IsNullOrEmpty(model.WorkpieceName) && x.WorkpieceName == model.WorkpieceName)
                || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public WorkpieceViewModel? Insert(WorkpieceBindingModel model)
        {
            model.Id = source.Workpieces.Count > 0 ? source.Workpieces.Max(x => x.Id) + 1 : 1;

            var newWorkpiece = Workpiece.Create(model);

            if (newWorkpiece == null)
            {
                return null;
            }
            source.Workpieces.Add(newWorkpiece);
            source.SaveWorkpieces();

            return newWorkpiece.GetViewModel;
        }

        public WorkpieceViewModel? Update(WorkpieceBindingModel model)
        {
            var workPiece = source.Workpieces.FirstOrDefault(x => x.Id == model.Id);

            if (workPiece == null)
            {
                return null;
            }
            workPiece.Update(model);
            source.SaveWorkpieces();

            return workPiece.GetViewModel;
        }

        public WorkpieceViewModel? Delete(WorkpieceBindingModel model)
        {
            var element = source.Workpieces.FirstOrDefault(x => x.Id == model.Id);

            if (element != null)
            {
                source.Workpieces.Remove(element);
                source.SaveWorkpieces();

                return element.GetViewModel;
            }

            return null;
        }
    }
}
