﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyListImplement.Implements
{
    public class WorkpieceStorage : IWorkpieceStorage
    {
        private readonly DataListSingleton _source;
        public WorkpieceStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public List<WorkpieceViewModel> GetFullList()
        {
            var result = new List<WorkpieceViewModel>();
            foreach (var Workpiece in _source.Workpieces)
            {
                result.Add(Workpiece.GetViewModel);
            }
            return result;
        }
        public List<WorkpieceViewModel> GetFilteredList(WorkpieceSearchModel model)
        {
            var result = new List<WorkpieceViewModel>();
            if (string.IsNullOrEmpty(model.WorkpieceName))
            {
                return result;
            }
            foreach (var Workpiece in _source.Workpieces)
            {
                if (Workpiece.WorkpieceName.Contains(model.WorkpieceName))
                {
                    result.Add(Workpiece.GetViewModel);
                }
            }
            return result;
        }
        public WorkpieceViewModel? GetElement(WorkpieceSearchModel model)
        {
            if (string.IsNullOrEmpty(model.WorkpieceName) && !model.Id.HasValue)
            {
                return null;
            }
            foreach (var Workpiece in _source.Workpieces)
            {
                if ((!string.IsNullOrEmpty(model.WorkpieceName) &&
               Workpiece.WorkpieceName == model.WorkpieceName) ||
                (model.Id.HasValue && Workpiece.Id == model.Id))
                {
                    return Workpiece.GetViewModel;
                }
            }
            return null;
        }
        public WorkpieceViewModel? Insert(WorkpieceBindingModel model)
        {
            model.Id = 1;
            foreach (var Workpiece in _source.Workpieces)
            {
                if (model.Id <= Workpiece.Id)
                {
                    model.Id = Workpiece.Id + 1;
                }
            }
            var newWorkpiece = Workpiece.Create(model);
            if (newWorkpiece == null)
            {
                return null;
            }
            _source.Workpieces.Add(newWorkpiece);
            return newWorkpiece.GetViewModel;
        }
        public WorkpieceViewModel? Update(WorkpieceBindingModel model)
        {
            foreach (var Workpiece in _source.Workpieces)
            {
                if (Workpiece.Id == model.Id)
                {
                    Workpiece.Update(model);
                    return Workpiece.GetViewModel;
                }
            }
            return null;
        }
        public WorkpieceViewModel? Delete(WorkpieceBindingModel model)
        {
            for (int i = 0; i < _source.Workpieces.Count; ++i)
            {
                if (_source.Workpieces[i].Id == model.Id)
                {
                    var element = _source.Workpieces[i];
                    _source.Workpieces.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}